import datetime

import redis
from flask import (
    Blueprint,
    abort,
    current_app,
    jsonify,
    redirect,
    render_template,
    request,
    url_for,
)
from werkzeug import cached_property, url_quote_plus, url_unquote_plus

from .utils import (
    _get_db_details,
    _get_key_details,
    _get_key_info,
    VALUE_SETTERS,
    _decode_bytes,
    _update_config,
)
from .constant import BADGE_CLASS, INFO_GROUPS, CONFIG

module = Blueprint(
    "redisboard",
    __name__,
    template_folder="templates/redisboard",
    static_folder="static",
)


class RedisServer:
    @cached_property
    def connection(self):
        return redis.Redis(
            host=current_app.config["REDIS_HOST"],
            port=current_app.config["REDIS_PORT"],
            password=current_app.config["REDIS_PASSWORD"],
            unix_socket_path=current_app.config["REDIS_UNIX_SOCKET_PATH"],
            socket_timeout=current_app.config["REDISBOARD_SOCKET_TIMEOUT"],
            socket_connect_timeout=current_app.config[
                "REDISBOARD_SOCKET_CONNECT_TIMEOUT"
            ],
            socket_keepalive=current_app.config["REDISBOARD_SOCKET_KEEPALIVE"],
            socket_keepalive_options=current_app.config[
                "REDISBOARD_SOCKET_KEEPALIVE_OPTIONS"
            ],
        )

    @cached_property
    def info(self):
        pipe = self.connection.pipeline()
        for part in INFO_GROUPS:
            pipe.info(part)
        results = pipe.execute()
        return dict(zip(INFO_GROUPS, results))

    @property
    def keyspace(self):
        return self.connection.info("Keyspace")

    @property
    def commandstats(self):
        return self.connection.info("Commandstats")

    @property
    def databases(self):
        return [item[2:] for item in self.keyspace.keys()]

    def slowlog_get(self, limit=None):
        try:
            count = limit if limit else current_app.config["REDISBOARD_SLOWLOG_LEN"]
            for slowlog in self.connection.slowlog_get(count):
                yield dict(
                    id=slowlog["id"],
                    ts=datetime.datetime.fromtimestamp(slowlog["start_time"]),
                    duration=slowlog["duration"] // 1000,
                    command=_decode_bytes(slowlog["command"]),
                )

        except redis.exceptions.ConnectionError:
            pass


server = RedisServer()


@module.context_processor
def inject_param():
    return {"databases": server.databases}


@module.errorhandler(Exception)
def handle_exception(error):
    return jsonify({"code": 999, "error": str(error)})


@module.route("/")
def home():
    return redirect(url_for("redisboard.info"))


@module.route("/info/")
def info():
    return render_template(
        "serverinfo.html",
        basic_info=server.info,
        keyspace=server.keyspace,
        cmdstats=server.commandstats,
        slowlog=server.slowlog_get(),
    )


@module.route("/config/", methods=["GET", "POST"])
def config():
    conn = server.connection
    if request.method == "POST":
        value = ""
        if "value" in request.form:
            value = request.form.get("value")
        elif "value[]" in request.form:
            value = "".join(request.form.getlist("value[]"))
        try:
            conn.config_set(request.form.get("name"), value)
        except Exception as e:
            return jsonify({"code": 999, "error": str(e)})
        return jsonify({"code": 0})
    config_value = conn.config_get()
    config_file = server.info["Server"].get("config_file")
    _update_config(CONFIG, config_value)
    return render_template("config.html", config_file=config_file, config=CONFIG)


@module.route("/db/")
@module.route("/db/<db>/")
def db_detail(db=0):
    db_detail = server.keyspace.get(f"db{db}", dict())
    # avoid same name with dict.keys()
    db_detail["_keys"] = db_detail["keys"] if "keys" in db_detail else 0
    cursor = request.args.get("cursor", type=int, default=0)
    keypattern = request.args.get("keypattern", default="")
    # when search, use bigger paginate number
    count = 1000 if keypattern else 20
    db_detail.update(
        _get_db_details(
            server.connection, db, cursor=cursor, keypattern=keypattern, count=count
        )
    )
    return render_template(
        "database.html",
        db_detail=db_detail,
        db=db,
        count=count,
        badge_class=BADGE_CLASS,
        keypattern=keypattern,
    )


@module.route("/db/<db>/addkey", methods=["POST"])
def add_key(db):
    conn = server.connection
    conn.execute_command("SELECT", db)
    keyname = request.form.get("keyname")
    typex = request.form.get("type")
    index = request.form.get("index")
    value = request.form.get("value")
    set_method = VALUE_SETTERS.get(typex)
    set_method(conn, keyname, index, value)
    return redirect(url_for("redisboard.db_detail", db=db))


@module.route("/db/<db>/batchttl", methods=["POST"])
def batch_set_ttl(db):
    conn = server.connection
    conn.execute_command("SELECT", db)
    pipe = conn.pipeline()
    keys = request.json.get("keys", [])
    ttl = int(request.json.get("ttl", -1))
    for key in keys:
        if ttl <= 0:
            pipe.persist(key)
        else:
            pipe.expire(key, ttl)
    pipe.execute()
    return jsonify({"code": 0, "data": url_for("redisboard.db_detail", db=db)})


@module.route("/db/<db>/batchdel", methods=["POST"])
def batch_delete_keys(db):
    conn = server.connection
    conn.execute_command("SELECT", db)
    keys = request.json.get("keys", [])
    conn.delete(*keys)
    return jsonify({"code": 0, "data": url_for("redisboard.db_detail", db=db)})


@module.route("/db/<db>/flush", methods=["DELETE"])
def db_flush(db):
    conn = server.connection
    conn.execute_command("SELECT", db)
    conn.flushdb()
    return jsonify({"data": "ok"})


@module.route("/db/<db>/key/<key>/del", methods=["DELETE"])
def key_delete(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    key = url_unquote_plus(key)
    conn.delete(key)
    return jsonify({"data": "ok"})


@module.route("/db/<db>/<key>/rename", methods=["POST"])
def key_rename(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    key = url_unquote_plus(key)
    new_name = request.form["keyname"]
    conn.rename(key, new_name)
    return jsonify(
        {
            "code": 0,
            "data": url_for(
                "redisboard.key_detail", db=db, key=url_quote_plus(new_name)
            ),
        }
    )


@module.route("/db/<db>/<key>/ttl", methods=["POST"])
def key_set_ttl(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    ttl = request.form.get("ttl", type=int)
    if ttl <= 0:
        conn.persist(ori_key)
    else:
        conn.expire(ori_key, ttl)
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>/list_add", methods=["POST"])
def list_add_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    position = request.form.get("position", type=int)
    if position == 0:
        conn.lpush(ori_key, request.form["value"])
    elif position == -1:
        conn.rpush(ori_key, request.form["value"])
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>/list_edit", methods=["POST"])
def list_edit_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    index = request.form.get("name", type=int)
    conn.lset(ori_key, index, request.form.get("value"))
    return jsonify({"code": 0})


@module.route("/db/<db>/<key>/list_rem", methods=["POST"])
def list_rem_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    count = request.form.get("count", type=int, default=1)
    conn.lrem(ori_key, count, request.form["value"])
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>/hash_add", methods=["POST"])
def hash_add_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    index = request.form.get("index", "")
    exists = conn.hexists(ori_key, index)
    if exists:
        return jsonify({"code": 1, "error": "can`t add value to an exist key!"})
    else:
        conn.hset(ori_key, index, request.form.get("value"))
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>/hash_edit", methods=["POST"])
def hash_edit_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    conn.hset(ori_key, request.form.get("name"), request.form.get("value"))
    return jsonify({"code": 0})


@module.route("/db/<db>/<key>/hash_rem", methods=["POST"])
def hash_rem_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    conn.hdel(ori_key, request.form["index"])
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>/set_add", methods=["POST"])
def set_add_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    value = request.form.get("value", "")
    value = [item.strip() for item in value.split(",")]
    result = conn.sadd(ori_key, *value)
    # TODO response how many successed operate
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>/set_rem", methods=["POST"])
def set_rem_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    value = request.form.get("value", "")
    value = [item.strip() for item in value.split(",")]
    conn.srem(ori_key, *value)
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>/zset_edit", methods=["POST"])
def zset_edit_score(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    maps = {request.form.get("name"): request.form.get("value", type=float)}
    conn.zadd(ori_key, maps)
    return jsonify({"code": 0})


@module.route("/db/<db>/<key>/zset_add", methods=["POST"])
def zset_add_value(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    conn.zadd(
        ori_key, {request.form.get("member"): request.form.get("score", type=float)}
    )
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>/zset_rem", methods=["POST"])
def zset_rem_member(db, key):
    conn = server.connection
    conn.execute_command("SELECT", db)
    ori_key = url_unquote_plus(key)
    member = request.form.get("member", "")
    if member:
        members = [item.strip() for item in member.split(",")]
        conn.zrem(ori_key, *members)
    else:
        score_min, score_max = (
            request.form.get("min", type=float),
            request.form.get("max", type=float),
        )
        conn.zremrangebyscore(ori_key, score_min, score_max)
    return jsonify(
        {"code": 0, "data": url_for("redisboard.key_detail", db=db, key=key)}
    )


@module.route("/db/<db>/<key>", methods=["GET", "POST"])
def key_detail(db, key):
    conn = server.connection
    key = url_unquote_plus(key)
    if request.method == "POST":
        conn.set(key, request.form["value"])
    key_details = _get_key_details(conn, db, key)
    return render_template(
        f"keydetail/{key_details['type']}.html", key_details=key_details, db=db
    )




from werkzeug import url_quote_plus

from .redisboard import module


class RedisBoardExtension:
    def __init__(self, app=None):
        self.app = app

        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        for k, v in self._default_config().items():
            app.config.setdefault(k, v)
        app.jinja_env.filters["quote_plus"] = url_quote_plus
        app.register_blueprint(module, url_prefix="/redisboard")

    def _default_config(self):
        return {
            "REDIS_HOST": "localhost",
            "REDIS_PORT": 6379,
            "REDIS_PASSWORD": None,
            "REDIS_UNIX_SOCKET_PATH": None,
            "REDISBOARD_SOCKET_TIMEOUT": None,
            "REDISBOARD_SOCKET_CONNECT_TIMEOUT": None,
            "REDISBOARD_SOCKET_KEEPALIVE": None,
            "REDISBOARD_SOCKET_KEEPALIVE_OPTIONS": None,
            "REDISBOARD_SLOWLOG_LEN": 100,
        }



import datetime

from flask import abort
import redis


def zset_getter(conn, key):
    result = conn.zrange(key, start=0, end=-1, withscores=True)
    result = [(_decode_bytes(item[0]), item[1]) for item in result]
    return result


def set_getter(conn, key):
    return ",   ".join([_decode_bytes(value) for value in conn.smembers(key)])


def list_getter(conn, key):
    return [
        (index, _decode_bytes(value))
        for index, value in enumerate(conn.lrange(key, start=0, end=1000))
    ]


def hash_getter(conn, key):
    return [(_decode_bytes(k), _decode_bytes(v)) for k, v in conn.hgetall(key).items()]


VALUE_GETTERS = {
    "list": list_getter,
    "string": lambda conn, key, *args: _decode_bytes(conn.get(key)),
    "set": set_getter,
    "zset": zset_getter,
    "hash": hash_getter,
    "n/a": lambda conn, key, *args: (),
}

LENGTH_GETTERS = {
    b"list": lambda conn, key: conn.llen(key),
    b"string": lambda conn, key: conn.strlen(key),
    b"set": lambda conn, key: conn.scard(key),
    b"zset": lambda conn, key: conn.zcount(key, "-inf", "+inf"),
    b"hash": lambda conn, key: conn.hlen(key),
}


def list_setter(conn, key, index, value):
    value = [item.strip() for item in value.split(",")]
    conn.lpush(key, *value)


def set_setter(conn, key, index, value):
    value = [item.strip() for item in value.split(",")]
    conn.sadd(key, *value)


VALUE_SETTERS = {
    "list": list_setter,
    "string": lambda conn, key, index, value: conn.set(key, value),
    "set": set_setter,
    "zset": lambda conn, key, score, member: conn.zadd(key, {member: float(score)}),
    "hash": lambda conn, key, index, value: conn.hset(key, index, value),
}


def _decode_bytes(value):
    if isinstance(value, bytes):
        try:
            result = value.decode()
        except UnicodeDecodeError:
            result = value
    else:
        result = value
    return result


def ttl_formatter(seconds):
    # redis-py under 3.0, if not set expired, the ttl return none.
    if seconds == -1 or seconds is None:
        return "forever"
    ttl = datetime.timedelta(seconds=seconds)
    mm, ss = divmod(ttl.seconds, 60)
    hh, mm = divmod(mm, 60)
    if ttl.days:
        return f"{ttl.days}day,{hh}hour,{mm}min,{ss}seconds"
    else:
        return f"{hh}hour,{mm}min,{ss}seconds"


def _get_db_details(conn, db, cursor=0, keypattern=None, count=20):
    conn.execute_command("SELECT", db)
    keypattern = f"*{keypattern}*" if keypattern else None
    cursor, keys = conn.scan(cursor=cursor, match=keypattern, count=count)
    key_details = [_get_key_info(conn, key.decode()) for key in keys]
    return dict(key_details=key_details, cursor=cursor)


def _get_key_details(conn, db, key):
    conn.execute_command("SELECT", db)
    details = _get_key_info(conn, key)
    details["db"] = db
    details["data"] = VALUE_GETTERS[details["type"]](conn, key)
    return details


def _get_key_info(conn, key):
    obj_type = conn.type(key)
    if obj_type == b"none":
        abort(404)
    pipe = conn.pipeline()
    try:
        pipe.object("REFCOUNT", key)
        pipe.object("ENCODING", key)
        pipe.object("IDLETIME", key)
        LENGTH_GETTERS[obj_type](pipe, key)
        pipe.ttl(key)

        refcount, encoding, idletime, obj_length, obj_ttl = pipe.execute()
    except redis.exceptions.ResponseError as exc:
        return {
            "type": obj_type,
            "name": key,
            "length": "n/a",
            "error": str(exc),
            "ttl": "n/a",
            "refcount": "n/a",
            "encoding": "n/a",
            "idletime": "n/a",
        }
    return {
        "type": _decode_bytes(obj_type),
        "name": key,
        "length": obj_length,
        "ttl": ttl_formatter(obj_ttl),
        "refcount": refcount,
        "encoding": _decode_bytes(encoding),
        "idletime": idletime,
    }


def _update_config(config_constants, config_value):
    for config_part, name in config_constants:
        for k, v in config_part.items():
            config_part[k]["value"] = config_value.get(k)




from collections import OrderedDict

BADGE_CLASS = {
    "string": "badge-info",
    "list": "badge-success",
    "set": "badge-warning",
    "hash": "badge-dark",
    "zset": "badge-light",
}

INFO_GROUPS = [
    "Server",
    "Clients",
    "Memory",
    "Persistence",
    "Stats",
    "Replication",
    "Cpu",
    "Cluster",
]


logfile_desc = "You can specify the log file name.An empty string can be used to force Redis to log on the standard output. Note that if you use standard output for logging but daemonize, logs will be sent to /dev/null."
pidfile_desc = "When redis is running as daemon it creates a pid file. Redis writes it where specified at startup and removes it at exit. You can set the path for pid file."
databases_desc = "Set the number of databases.The default database is DB 0, you can select a different one on a per-connection basis using SELECT <dbid> where dbid is a number between 0 and 'databases'-1."
daemonize_desc = "Use 'yes' if you want Redis to run as a daemon. By default Redis does not run as a daemon. Redis will write a pid file in /var/run/redis.pid when daemonized."
loglevel_desc = "This config specifies the verbosity level.Debug is useful for development and testing as a lot of information is given.Verbose is selected to logs many rarely used info.Notice is used in production because it is moderately verbose.Warning as the name specifies logs only the very important messages"
supervised_desc = "If you run Redis from upstart or systemd, Redis can interact with your supervision tree.If 'no' is selected then there will be no supervision interaction.If upstart is selected - signal upstart by putting Redis into SIGSTOP mode.If systemd is selected -signal systemd by writing READY=1 to $NOTIFY_SOCKET and If it is set to auto-detect upstart or systemd method based on UPSTART_JOB or NOTIFY_SOCKET environment variables."
syslog_fac_desc = (
    "Specifies the syslog facility. Must be USER or between LOCAL0-LOCAL7."
)


GENERAL_CONFIG = OrderedDict(
    {
        "logfile": {"desc": logfile_desc, "type": "text", "can_edit": True},
        "pidfile": {"desc": pidfile_desc, "type": "text", "can_edit": True},
        "databases": {"desc": databases_desc, "can_edit": False},
        "daemonize": {"desc": daemonize_desc, "can_edit": False},
        "loglevel": {"desc": loglevel_desc, "type": "select", "can_edit": True},
        "supervised": {"desc": supervised_desc, "can_edit": False},
        "syslog-facility": {"desc": syslog_fac_desc, "type": "text", "can_edit": True},
    }
)


unixsocket_desc = "Specify the path for the Unix socket that will be used to listen for incoming connections.There is no default, so Redis will not listen on a unix socket when not specified."
timeout_desc = (
    "Close the connection after a client is idle for N seconds (0 to disable)."
)
port_desc = "Accept connections on the specified port, default is 6379 (IANA #815344). If port 0 is specified Redis will not listen on a TCP socket."
tcp_backlog_desc = "In high requests-per-second environments you need an high backlog in order to avoid slow clients connections issues. Note that the Linux kernel will silently truncate it to the value of /proc/sys/net/core/somaxconn so make sure to raise both the value of somaxconn and tcp_max_syn_backlog in order to get the desired effect."
tcp_keepalive_desc = "In absence of communication redis sends ACKs to clients to check for dead peers.This config specifies the time period used to send ACKs. To close the connection double the specified time is needed.Default is set to 300."
protected_mode_desc = "Protected mode is a layer of security protection.By default protected mode is enabled.You should disable it only if you are sure you want clients from other hosts to connect to Redis even if no authentication is configured, nor a specific set of interfaces are explicitly listed using the bind directive."
unixsocketperm_desc = "Specify the path for the Unix socket that will be used to listen for incoming connections. There is no default, so Redis will not listen on a unix socket when not specified."
bind_desc = "By default, if no bind configuration directive is specified, Redis listens for connections from all the network interfaces available on the server.It is possible to listen to just one or multiple selected interfaces using the bind configuration directive, followed by one or more IP addresses."

NETWORK_CONFIG = OrderedDict(
    {
        "unixsocket": {"desc": unixsocket_desc, "can_edit": False},
        "timeout": {"desc": timeout_desc, "type": "number", "can_edit": True},
        "port": {"desc": port_desc, "can_edit": False},
        "tcp-backlog": {"desc": tcp_backlog_desc, "can_edit": False},
        "tcp-keepalive": {
            "desc": tcp_keepalive_desc,
            "type": "number",
            "can_edit": True,
        },
        "protected-mode": {
            "desc": protected_mode_desc,
            "type": "select",
            "can_edit": True,
        },
        "unixsocketperm ": {"desc": unixsocketperm_desc, "can_edit": False},
        "bind ": {"desc": bind_desc, "can_edit": False},
    }
)

dbfilename_desc = "The filename where to dump the database."
stop_writes_on_bgsave_error_desc = "By default Redis will stop accepting writes if RDB snapshots are enabled and the latest background save failed. This will make you aware that data is not persisting on disk properly, otherwise chances are that no one will notice and some disaster will happen.If the background saving process will start working again Redis will automatically allow writes again.You can disable this so that Redis continue to work as usual even if there are problems with the disk."
rdbcompression_desc = "This Compresses string objects using LZF when dump .rdb databases.By default it is set to yes.If you want to save some CPU then set it to no."
rdbchecksum_desc = "Since version 5 of RDB a CRC64 checksum is placed at the end of the file. This makes the format more resistant to corruption but there is a performance hit to pay (around 10%) when saving and loading RDB files, so you can disable it for maximum performances.RDB files created with checksum disabled have a checksum of zero that will tell the loading code to skip the check."
dir_desc = "The working directory.The DB will be written inside this directory, with the filename specified above using the 'dbfilename' configuration directive.The Append Only File will also be created inside this directory."
save_desc = "Used for saving db on disk. The format is 'save <seconds><changes>'.Will save the DB if both the given number of seconds and the given number of write operations against the DB occurred."

SNAPSHOTTING_CONFIG = OrderedDict(
    {
        "dbfilename": {"desc": dbfilename_desc, "type": "text", "can_edit": True},
        "stop-writes-on-bgsave-error": {
            "desc": stop_writes_on_bgsave_error_desc,
            "type": "select",
            "can_edit": True,
        },
        "rdbcompression": {
            "desc": rdbcompression_desc,
            "type": "select",
            "can_edit": True,
        },
        "rdbchecksum": {"desc": rdbchecksum_desc, "can_edit": False},
        "dir": {"desc": dir_desc, "type": "text", "can_edit": True},
        "save": {"desc": save_desc, "type": "text", "can_edit": True},
    }
)

wait_to_add = "wait to add"


REPLICATION_CONFIG = OrderedDict(
    {
        "masterauth": {"desc": wait_to_add, "can_edit": False},
        "slave-announce-ip ": {"desc": wait_to_add, "type": "text", "can_edit": True},
        "repl-ping-slave-period ": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "repl-timeout": {"desc": wait_to_add, "type": "number", "can_edit": True},
        "repl-backlog-size": {"desc": wait_to_add, "type": "number", "can_edit": True},
        "repl-backlog-ttl": {"desc": wait_to_add, "type": "number", "can_edit": True},
        "slave-priority": {"desc": wait_to_add, "type": "number", "can_edit": True},
        "slave-announce-port": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "min-slaves-to-write": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "min-slaves-max-lag": {"desc": wait_to_add, "type": "number", "can_edit": True},
        "repl-diskless-sync-delay": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "slave-serve-stale-data": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
        "slave-read-only": {"desc": wait_to_add, "type": "select", "can_edit": True},
        "repl-disable-tcp-nodelay": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
        "repl-diskless-sync": {"desc": wait_to_add, "type": "select", "can_edit": True},
        "slaveof": {"desc": wait_to_add, "can_edit": False},
    }
)

SECURITY_CONFIG = OrderedDict({"requirepass": {"desc": wait_to_add, "can_edit": False}})

CLIENTS_CONFIG = OrderedDict(
    {"maxclients": {"desc": wait_to_add, "type": "number", "can_edit": True}}
)

MEMORY_CONFIG = OrderedDict(
    {
        "maxmemory": {"desc": wait_to_add, "type": "number", "can_edit": True},
        "maxmemory-samples": {"desc": wait_to_add, "type": "number", "can_edit": True},
        "maxmemory-policy": {"desc": wait_to_add, "type": "select", "can_edit": True},
    }
)

LAZY_FREEING_CONFIG = OrderedDict(
    {
        "lazyfree-lazy-eviction": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
        "lazyfree-lazy-expire": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
        "lazyfree-lazy-server-del": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
        "slave-lazy-flush": {"desc": wait_to_add, "type": "select", "can_edit": True},
    }
)

APPEND_ONLY_MODE_CONFIG = OrderedDict(
    {
        "auto-aof-rewrite-percentage": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "auto-aof-rewrite-min-size": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "no-appendfsync-on-rewrite": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
        "aof-load-truncated": {"desc": wait_to_add, "type": "select", "can_edit": True},
        "aof-use-rdb-preamble": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
        "appendfsync": {"desc": wait_to_add, "type": "select", "can_edit": True},
        "appendonly": {"desc": wait_to_add, "type": "select", "can_edit": True},
    }
)

LUA_SCRIPTING_CONFIG = OrderedDict(
    {"lua-time-limit": {"desc": wait_to_add, "type": "number", "can_edit": True}}
)

REDIS_CLUSTER_CONFIG = OrderedDict(
    {
        "cluster-node-timeout": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "cluster-migration-barrier": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "cluster-slave-validity-factor": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "cluster-require-full-coverage": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
    }
)

CLUSTER_DOCKER_NAT_CONFIG = OrderedDict(
    {
        "cluster-announce-ip": {"desc": wait_to_add, "type": "text", "can_edit": True},
        "cluster-announce-port": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "cluster-announce-bus-port ": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
    }
)

SLOWLOG_CONFIG = OrderedDict(
    {
        "slowlog-log-slower-than": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "slowlog-max-len": {"desc": wait_to_add, "type": "number", "can_edit": True},
    }
)

LATENCY_MONITOR_CONFIG = OrderedDict(
    {
        "latency-monitor-threshold": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        }
    }
)

EVENT_NOTIFICATION_CONFIG = OrderedDict(
    {
        "notify-keyspace-events": {
            "desc": wait_to_add,
            "type": "checklist",
            "can_edit": True,
        }
    }
)

DEFRAGMENTATION_CONFIG = OrderedDict(
    {
        "active-defrag-threshold-lower": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "active-defrag-threshold-upper": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "active-defrag-ignore-bytes": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "active-defrag-cycle-min": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "active-defrag-cycle-max": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "activedefrag": {"desc": wait_to_add, "type": "select", "can_edit": True},
    }
)

ADVANCED_CONFIG = OrderedDict(
    {
        "hash-max-ziplist-entries": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "hash-max-ziplist-value": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "list-max-ziplist-size": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "list-compress-depth": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "set-max-intset-entries": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "zset-max-ziplist-entries": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "zset-max-ziplist-value": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "hll-sparse-max-bytes": {
            "desc": wait_to_add,
            "type": "number",
            "can_edit": True,
        },
        "hz": {"desc": wait_to_add, "type": "number", "can_edit": True},
        "activerehashing": {"desc": wait_to_add, "type": "select", "can_edit": True},
        "aof-rewrite-incremental-fsync": {
            "desc": wait_to_add,
            "type": "select",
            "can_edit": True,
        },
        "client-output-buffer-limit": {
            "desc": wait_to_add,
            "type": "text",
            "can_edit": True,
        },
    }
)


CONFIG = (
    (NETWORK_CONFIG, "NETWORK"),
    (GENERAL_CONFIG, "GENERAL"),
    (SNAPSHOTTING_CONFIG, "SNAPSHOTTING"),
    (REPLICATION_CONFIG, "REPLICATION"),
    (SECURITY_CONFIG, "SECURITY"),
    (CLIENTS_CONFIG, "CLIENTS"),
    (MEMORY_CONFIG, "MEMORY MANAGEMENT"),
    (LAZY_FREEING_CONFIG, "LAZY FREEING"),
    (APPEND_ONLY_MODE_CONFIG, "APPEND ONLY MODE"),
    (LUA_SCRIPTING_CONFIG, "LUA SCRIPTING"),
    (REDIS_CLUSTER_CONFIG, "REDIS CLUSTER"),
    (CLUSTER_DOCKER_NAT_CONFIG, "CLUSTER DOCKER/NAT"),
    (SLOWLOG_CONFIG, "SLOWLOG"),
    (LATENCY_MONITOR_CONFIG, "LATENCY MONITOR"),
    (EVENT_NOTIFICATION_CONFIG, "EVENT NOTIFICATION"),
    (DEFRAGMENTATION_CONFIG, "DEFRAGMENTATION"),
    (ADVANCED_CONFIG, "ADVANCED CONFIG"),
)



"""
    Flask-Redisboard
    ~~~~~~~~~~~~~~
    A flask extension to support user view and manage redis with beautiful interface.
    :author: hjlarry<ultrahe@gmail.com>
    :copyright: (c) 2019 by hjlarry.
    :license: MIT, see LICENSE for more details.
"""
from os import path
from codecs import open
from setuptools import setup

basedir = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(basedir, "README.md"), encoding="utf-8") as f:
    long_description = f.read()


setup(
    name="Flask-Redisboard",
    version="0.1.3",
    url="https://github.com/hjlarry/flask-redisboard",
    license="MIT",
    author="hjlarry",
    author_email="ultrahe@gmail.com",
    description="A flask extension to support user view and manage redis with beautiful interface.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    platforms="any",
    packages=["flask_redisboard"],
    zip_safe=False,
    include_package_data=True,
    install_requires=["Flask", "redis"],
    extras_require={"dev": ["black"]},
    keywords="flask extension development redis",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
